﻿using System;
using System.Net.Sockets;
using SWConfigDataClientLib;
using SWConfigDataClientLib.Proxies.PhoneClient;
using SWConfigDataSharedLib.Exceptions;
using SWConfigDataSharedLib.Tracing;

namespace $safeprojectname$
{
    /// <summary>
    /// CDS Client Library
    /// </summary>
    public class CDSLib : IDisposable
    {
        #region CDS Client Members
        private LibManager m_LibManager;
        private PhoneClientFacade m_PhoneClientFacade;
        private bool m_Connected;
        #endregion

        #region CDS Client Tracing
        // Init user defined trace module named "CDSlib"
        // See App.config for trace module configuration
        private static STraceInfo ms_Tracer = new STraceInfo(new System.Diagnostics.TraceSwitch("CDSlib", "CDSlib", "2"), typeof(CDSLib));
        #endregion
                
        //------------------------------------------------------------------------------------

        #region Properties

        public string UserName
        {
            get;
            set;
        }

        public string Password
        {
            get;
            set;
        }

        public string ServerName
        {
            get;
            set;
        }

        public SProxyObject.AuthenticationMode AuthenticationMode
        {
            get;
            set;
        }

        private int UserID
        {
            get;
            set;
        }

        private bool IsSystemAdmin
        {
            get;
            set;
        }

        #endregion

        //------------------------------------------------------------------------------------

        #region CTor & DTor

        /// <summary>
        /// Creates a new CDS object (Windows Authentication)
        /// </summary>        
        public CDSLib()
        {
            this.UserName = String.Empty;
            this.Password = String.Empty;
            this.ServerName = String.Empty;
            this.AuthenticationMode = SProxyObject.AuthenticationMode.TrustedLocal;
            InitLibManager();
        }

        /// <summary>
        /// Creates a new CDS object (Windows Authentication)
        /// </summary>     
        /// <param name="serverName">SwyxServer Address (FQDN or IP)</param>
        public CDSLib(string serverName)
        {
            this.UserName = String.Empty;
            this.Password = String.Empty;
            this.ServerName = serverName;
            this.AuthenticationMode = SProxyObject.AuthenticationMode.Trusted;
            InitLibManager();
        }

        /// <summary>
        /// Creates a new CDS object (UsernamePassword Authentication)
        /// </summary>
        /// <param name="userName">Username (SwyxIt Login)</param>
        /// <param name="password">Password (SwyxIt Login)</param>
        /// <param name="serverName">SwyxServer Address (FQDN or IP)</param>
        public CDSLib(string userName, string password, string serverName)
        {
            if (String.IsNullOrEmpty(userName) || String.IsNullOrEmpty(password))
                throw new ArgumentException("Invalid parameters");

            this.UserName = userName;
            this.Password = password;
            this.ServerName = serverName;
            this.AuthenticationMode = SProxyObject.AuthenticationMode.UsernamePassword;

            InitLibManager();
        }

        /// <summary>
        /// Init LibManager with given values
        /// </summary>
        private void InitLibManager()
        {
            m_Connected = false;
            this.IsSystemAdmin = false;
            this.UserID = -1;

            if (m_LibManager == null)
                m_LibManager = new LibManager();

            if (this.AuthenticationMode == SProxyObject.AuthenticationMode.UsernamePassword && !this.UserName.StartsWith("ippbx:", StringComparison.OrdinalIgnoreCase))
            {
                this.UserName = "ippbx:" + this.UserName;
            }

            if (String.IsNullOrEmpty(this.ServerName))
            {
                this.ServerName = "127.0.0.1";
            }

            m_LibManager.Username = this.UserName;
            m_LibManager.Password = this.Password;
            m_LibManager.AuthenticationMode = this.AuthenticationMode;
            m_LibManager.BaseProtocol = SProxyObject.BaseProtocol.TCP;
            m_LibManager.WSBaseUrl = this.ServerName;
        }

        //------------------------------------------------------------------------------------

        /// <summary>
        /// Destructor
        /// </summary>
        ~CDSLib()
        {
            this.Dispose(false);
        }
            
        #endregion

        //------------------------------------------------------------------------------------

        #region IDisposable Members

        /// <summary>
        /// Performs application-defined tasks associated with freeing,
        /// releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing,
        /// releasing, or resetting unmanaged resources.
        /// </summary>
        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
                // Check if connection was disconnected.
                this.Disconnect();
            }
        }

        #endregion

        //------------------------------------------------------------------------------------

        #region CDS Connect & Disconnect

        /// <summary>
        /// Creates a new CDS connection
        /// </summary>
        public bool Connect()
        {
            if (IsConnected())
                return true;

            try
            {
                // Get Phone Client Facade
                m_PhoneClientFacade = m_LibManager.GetPhoneClientFacade();

                // Get the ID of the current user
                this.UserID = m_PhoneClientFacade.GetCurrentUserID();

                // Check for System Administrator rights
                this.IsSystemAdmin = m_PhoneClientFacade.HasRole(SWConfigDataClientLib.WSPhoneClientFacade.BuildInRoles.Admin);

                ms_Tracer.TraceVerbose("Connect", "UserName: {0} / UserID: {1} / IsSysAdmin: {2}", this.UserName, this.UserID, this.IsSystemAdmin);
                ms_Tracer.TraceInfo("Connect", "CDS Connection successfully establised ({0})", this.ServerName);
                m_Connected = true;
                return true;
            }
            catch (SocketException ex)
            {
                ms_Tracer.TraceException(ex, true);
            }
            catch (STimeoutException ex)
            {
                ms_Tracer.TraceException(ex, true);
            }
            catch (SSecurityAccessException ex)
            {
                ms_Tracer.TraceException(ex, true);
            }
            catch (Exception ex)
            {
                ms_Tracer.TraceException(ex, true);
            }

            m_Connected = false;
            ms_Tracer.TraceError("Connect", "CDS Connection failed ({0})", this.ServerName);

            return false;
        }

        //------------------------------------------------------------------------------------

        /// <summary>
        /// CDS objects will be cleaned up
        /// </summary>
        public bool Disconnect()
        {
            if (!IsConnected())
                return false;

            try
            {
                if (m_PhoneClientFacade != null)
                {
                    m_PhoneClientFacade.FreeForReuse();
                    m_PhoneClientFacade = null;
                }

                if (m_LibManager != null)
                {                    
                    m_LibManager.FreeForReuse();
                    m_LibManager.Dispose();
                    m_LibManager = null;
                }

                m_Connected = false;
                ms_Tracer.TraceInfo("DisconnectFromCDS", "Successfully disconnected from {0}", this.ServerName);
                return true;
            }            
            catch (Exception ex)
            {
                ms_Tracer.TraceException(ex, true);
            }
            ms_Tracer.TraceError("DisconnectFromCDS", "Process failed for server {0}", this.ServerName);
            return false;
        }

        //------------------------------------------------------------------------------------

        /// <summary>
        /// CDS already connected?
        /// This method does NOT write errors to the log file
        /// </summary>
        public bool IsConnected()
        {
            if (m_Connected)
                return true;
            else return false;
        }

        //------------------------------------------------------------------------------------

        /// <summary>
        /// CDS already connected?
        /// !!! This method contains error tracing !!!
        /// </summary>
        /// <param name="methodName">Name of the trigger method (used for tracing)</param>
        public bool IsConnected(string methodName)
        {
            if (m_Connected)
                return true;
            else
            {
                ms_Tracer.TraceError(methodName, "No CDS connection to {0}! Connection lost?", this.ServerName);
                return false;
            }
        }

        #endregion

        //------------------------------------------------------------------------------------

        #region Sample Methods

        /// <summary>
        /// Returns a UserPhoneCallListEntrySortableCollection for an ippbx user
        /// </summary>
        public UserPhoneCallListEntrySortableCollection GetCallerList(int userId)
        {
            ms_Tracer.TraceInfo("GetCallerList", "({0})", userId);

            if (!IsConnected("GetCallerList"))
                return null;

            try
            {
                if (m_PhoneClientFacade == null)
                    m_PhoneClientFacade = m_LibManager.GetPhoneClientFacade();

                return m_PhoneClientFacade.GetUserPhoneCallList(userId, false).CreateSortableFilterCollection();
            }            
            catch (Exception ex)
            {
                ms_Tracer.TraceException(ex, true);
            }
            return null;
        }

        //------------------------------------------------------------------------------------

        /// <summary>
        /// Returns a RedialItemEntrySortableCollection for an ippbx user
        /// </summary>
        public RedialItemEntrySortableCollection GetRedialList(int userId)
        {
            ms_Tracer.TraceInfo("GetRedialList", "({0})", userId);

            if (!IsConnected("GetRedialList"))
                return null;

            try
            {
                if (m_PhoneClientFacade == null)
                    m_PhoneClientFacade = m_LibManager.GetPhoneClientFacade();

                return m_PhoneClientFacade.GetRedialItems(userId).CreateSortableFilterCollection();
            }            
            catch (Exception ex)
            {
                ms_Tracer.TraceException(ex, true);
            }
            return null;
        }

        //------------------------------------------------------------------------------------

        /// <summary>
        /// Delete the Caller List of an ippbx user
        /// </summary>
        public bool ClearCallerList(int userId)
        {
            ms_Tracer.TraceInfo("ClearCallerList", "({0})", userId);

            if (!IsConnected("ClearCallerList"))
                return false;

            try
            {
                if (m_PhoneClientFacade == null)
                    m_PhoneClientFacade = m_LibManager.GetPhoneClientFacade();

                m_PhoneClientFacade.DeleteUserPhoneCalls(userId);
                return true;
            }
            catch (Exception ex)
            {
                ms_Tracer.TraceException(ex, true);
            }
            return false;
        }

        //------------------------------------------------------------------------------------

        /// <summary>
        /// Delete the Redial List of an ippbx user
        /// </summary>
        public bool ClearRedialList(int userId)
        {
            ms_Tracer.TraceInfo("ClearRedialList", "({0})", userId);

            if (!IsConnected("ClearRedialList"))
                return false;

            try
            {
                if (m_PhoneClientFacade == null)
                    m_PhoneClientFacade = m_LibManager.GetPhoneClientFacade();

                m_PhoneClientFacade.DeleteRedialItems(userId);
                return true;
            }
            catch (Exception ex)
            {
                ms_Tracer.TraceException(ex, true);
            }
            return false;
        }

        #endregion

        //------------------------------------------------------------------------------------

    }
}